@@ -626,9 +626,6 @@ on_udisks_client_changed (UDisksClient *client,
   GVfsUDisks2Volume *volume = GVFS_UDISKS2_VOLUME (user_data);
   MountData *data = NULL;
 
-  /* This is needed as gvfs_udisks2_volume_monitor_update may remove this volume. */
-  g_object_ref (volume);
-
   update_volume_on_event (volume);
 
   if (volume->mount_pending_op)
@@ -646,8 +643,6 @@ on_udisks_client_changed (UDisksClient *client,
           g_object_unref (cleartext_block);
         }
     }
-
-  g_object_unref (volume);
 }
 
 /* takes ownership of @mount_point if not NULL */
